@@ -1,4 +1,4 @@
-NumStarts<-function(p,gamma=0.99,eps=0.5){
+numStarts<-function(p,gamma=0.99,eps=0.5){
 	if(p>25)	stop("p too large.")
 	if(gamma>=1)	stop("gamma should be smaller than 1.")
 	if(gamma<=0)	stop("gamma should be larger than 0.")
@@ -8,9 +8,9 @@ NumStarts<-function(p,gamma=0.99,eps=0.5){
 	ord<-10^floor(log10(ns0))
 	max(100,ceiling(ns0/ord)*ord)
 }
-FastRCS<-function(x,y,nsamp=NULL,alpha=0.5,seed=1){#x<-x0;y<-y0;nsamp<-ns;alpha<-0.5
+FastRCS<-function(x,y,nSamp=NULL,alpha=0.5,seed=1){#x<-x0;y<-y0;nSamp<-ns;alpha<-0.5
 	k1<-25;k0<-25;J<-3;
-	m1<-"seed should be in [0,2**31]."
+	m1<-"seed should be an integer in [0,2**31]."
 	if(!is.null(seed)){
 		if(!is.finite(seed))		stop(m1)
 		if(!is.numeric(seed))		stop(m1)
@@ -31,7 +31,7 @@ FastRCS<-function(x,y,nsamp=NULL,alpha=0.5,seed=1){#x<-x0;y<-y0;nsamp<-ns;alpha<
 	p<-ncol(cx)
 	if(p<2)			stop("Univariate RCS is not implemented.")
 	if(p>25)		stop("FastRCS only works for dimensions <=25.")
-	if(is.null(nsamp)) 	nsamp<-NumStarts(p,eps=(1-alpha)) 
+	if(is.null(nSamp)) 	nSamp<-numStarts(p,eps=(1-alpha)) 
 	h<-quanf(n=n,p=p,alpha=alpha)
 	h0<-quanf(n=n,p=p,alpha=0.5);
 	Dp<-rep(1.00,n);
@@ -41,34 +41,39 @@ FastRCS<-function(x,y,nsamp=NULL,alpha=0.5,seed=1){#x<-x0;y<-y0;nsamp<-ns;alpha<
 	n2<-n1<-rep(0,h0);
 	icandid<-1:n-1
 	ni<-length(icandid)
-	fitd<-.C("fastrcs",as.integer(nrow(cx)),as.integer(ncol(cx)),as.integer(k0),as.single(cx),as.single(y),as.integer(k1),as.single(Dp),as.integer(nsamp),as.integer(J),as.single(objfunC),as.integer(seed),as.integer(icandid),as.integer(ni),as.integer(n1),as.integer(n2),as.integer(h0),PACKAGE="FastRCS")
+	fitd<-.C("fastrcs",
+		as.integer(nrow(cx)),	#1
+		as.integer(ncol(cx)),	#2
+		as.integer(k0),		#3
+		as.single(cx),		#4
+		as.single(y),		#5
+		as.integer(k1),		#6
+		as.single(Dp),		#7
+		as.integer(nSamp),	#8
+		as.integer(J),		#9
+		as.single(objfunC),	#10
+		as.integer(seed),	#11
+		as.integer(icandid),	#12
+		as.integer(ni),		#13
+		as.integer(n1),		#14
+		as.integer(n2),		#15
+		as.integer(h0),		#16
+		PACKAGE="FastRCS")
 	outd<-as.numeric(fitd[[7]])
 	if(is.nan(outd)[1])	stop("too many singular subsets encoutered!")
-	best<-fitd[[14]]#which(outd<=median(outd))
-	weit<-as.numeric((1:n)%in%best)
-	rawF<-lm(y~x,weights=weit)
-	rawF$resid<-y-cbind(1,x)%*%rawF$coef	
-	best<-which(abs(rawF$resid)<=quantile(abs(rawF$resid),h/n))
-	weit<-as.numeric((1:n)%in%best)
-	rawF<-lm(y~x,weights=weit);
-	rawF$best<-best
-	rawF$resid<-y-cbind(1,x)%*%rawF$coef
-	weit<-as.numeric(abs(rawF$resid)/median(abs(rawF$resid))/qnorm(1-alpha/2)<=sqrt(qchisq(0.975,df=1)))
-       	rewF<-lm(y~x,weights=weit);
-	rewF$best<-which(weit==1)	
-	rewF$resid<-y-cbind(1,x)%*%rewF$coef
 	best<-fitd[[15]]#which(outd<=median(outd))
 	weit<-as.numeric((1:n)%in%best)
-	rawC<-lm(y~x,weights=weit)
-	best<-which(abs(rawC$resid)<=quantile(abs(rawC$resid),h/n))
-	weit<-as.numeric((1:n)%in%best)
-	rawC<-lm(y~x,weights=weit);
-	rawC$best<-best
-	rawC$resid<-y-cbind(1,x)%*%rawC$coef
+	rawC<-lm(y~x,weights=weit)	
 	weit<-as.numeric(abs(rawC$resid)/median(abs(rawC$resid))/qnorm(1-alpha/2)<=sqrt(qchisq(0.975,df=1)))
-       	rewC<-lm(y~x,weights=weit);
-	rewC$best<-which(weit==1)	
-	rewC$resid<-y-cbind(1,x)%*%rewC$coef
-	list(outd=outd,alpha=alpha,nsamp=nsamp,obj=as.numeric(fitd[[10]]),raw=rawF,rew=rewF,rawC=rawC,rewC=rewC)
+       	FinalFit<-lm(y~x,weights=weit);
+	FinalFit$scale<-sqrt(sum(FinalFit$resid[weit==1]**2)/FinalFit$df)
+	A1 <- list(alpha=alpha,nSamp=nSamp,obj=as.numeric(fitd[[10]]), rawBest = fitd[[14]], rawDist=fitd[[7]],  
+	best = which(weit==1), coefficients = FinalFit$coef, fitted.values = FinalFit$fitted.values,residuals = FinalFit$residuals ,  rank = FinalFit$rank, weights= FinalFit$weights, df.residual = FinalFit$df.residual,scale=FinalFit$scale)
+	  class(A1) <-"FastRCS"
+	  return(A1)
 }
 quanf<-function(n,p,alpha)	return(floor(2*floor((n+p+1)/2)-n+2*(n-floor((n+p+1)/2))*alpha))
+plot.FastRCS<-function(x,col="black",pch=16,...){
+  plot(abs(x$resid)/x$scale,col=col,pch=pch,ylab="Robust standardized residuals",xlab="Index")
+  abline(h=sqrt(qchisq(0.975,df=1)),col="red",lty=2)
+}
